"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const inquirer = require("inquirer");
const aws_sdk_1 = require("aws-sdk");
class SearchPS extends command_1.Command {
    constructor() {
        super(...arguments);
        this.searchResults = [];
    }
    async getSearchResults(searchTerm, ssm, nextToken) {
        var _a, _b, _c;
        const { flags } = this.parse(SearchPS);
        const searchParams = {
            NextToken: nextToken,
            MaxResults: (_a = flags['max-results']) !== null && _a !== void 0 ? _a : 50,
        };
        if (!flags.all && searchTerm) {
            searchParams.ParameterFilters = [
                {
                    Key: 'Name',
                    Option: 'Contains',
                    Values: [searchTerm],
                },
            ];
        }
        const matches = await ssm.describeParameters(searchParams).promise();
        const parameters = (_c = (_b = matches.Parameters) === null || _b === void 0 ? void 0 : _b.map((parameter) => ({ name: parameter.Name }))) !== null && _c !== void 0 ? _c : [];
        this.searchResults.push(...parameters);
        if (matches.NextToken && (flags['no-limit'] || flags.all)) {
            await this.getSearchResults(searchTerm, ssm, matches.NextToken);
        }
    }
    async promptForParameters() {
        return inquirer.prompt([
            {
                name: 'parameters',
                message: 'Select your parameters',
                type: 'checkbox',
                choices: this.searchResults,
            },
        ]);
    }
    async run() {
        var _a, _b, _c, _d;
        const { flags } = this.parse(SearchPS);
        const ssm = new aws_sdk_1.SSM({ region: (_b = (_a = flags.region) !== null && _a !== void 0 ? _a : process.env.AWS_REGION) !== null && _b !== void 0 ? _b : 'us-east-1' });
        if (!((_c = ssm.config.credentials) === null || _c === void 0 ? void 0 : _c.sessionToken) && process.env.NODE_ENV !== 'test') {
            this.error('Please authenticate against AWS to use this tool');
        }
        let searchTerm;
        if (!flags.all) {
            searchTerm = (_d = flags.term) !== null && _d !== void 0 ? _d : (await cli_ux_1.default.prompt('What term do you want to search for?', { type: 'normal' }));
        }
        cli_ux_1.default.action.start('listing matching parameters');
        await this.getSearchResults(searchTerm, ssm);
        cli_ux_1.default.action.stop();
        if (this.searchResults && this.searchResults.length > 0) {
            let chosenParameters = await this.promptForParameters();
            while (chosenParameters.parameters.length === 0) {
                this.log('No parameters selected');
                chosenParameters = await this.promptForParameters();
            }
            const getParams = {
                Names: chosenParameters.parameters,
                WithDecryption: !flags['disable-decryption'],
            };
            cli_ux_1.default.action.start('retrieving parameters');
            const result = await ssm.getParameters(getParams).promise();
            cli_ux_1.default.action.stop();
            if (result.Parameters) {
                this.log('\n');
                cli_ux_1.default.table(result.Parameters, {
                    Name: {},
                    Value: {},
                    ARN: { extended: true },
                    Type: { extended: true },
                    Version: { extended: true },
                    LastModifiedDate: { extended: true },
                }, Object.assign({}, flags));
                return;
            }
        }
        this.warn('No Parameters found');
        this.exit(1);
    }
}
exports.default = SearchPS;
SearchPS.description = 'search parameters in AWS SSM Parameter Store';
SearchPS.examples = [`$ asap ps -t awesome-param`];
SearchPS.flags = {
    help: command_1.flags.help({ char: 'h' }),
    term: command_1.flags.string({ exclusive: ['all'], char: 't', description: 'term to search' }),
    region: command_1.flags.string({ char: 'r', description: 'AWS region override' }),
    'disable-decryption': command_1.flags.boolean({
        char: 'd',
        description: 'disable decryption of SecureString type params by default',
    }),
    'max-results': command_1.flags.integer({
        exclusive: ['no-limit', 'all'],
        char: 'n',
        description: 'limit for amount of results returned, default is 50',
    }),
    'no-limit': command_1.flags.boolean({
        exclusive: ['max-results', 'all'],
        description: 'retrieve every parameter for the search term',
    }),
    all: command_1.flags.boolean({
        exclusive: ['max-results', 'no-limit', 'term'],
        description: 'retrieve every parameter in the region',
    }),
    csv: command_1.flags.boolean({ exclusive: ['no-truncate'], description: 'output is csv format' }),
    extended: command_1.flags.boolean({ char: 'x', description: 'show extra columns' }),
    'no-truncate': command_1.flags.boolean({ exclusive: ['csv'], description: 'do not truncate output to fit screen' }),
    'no-header': command_1.flags.boolean({ exclusive: ['csv'], description: 'hide table header from output' }),
};
