"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const inquirer = require("inquirer");
const aws_sdk_1 = require("aws-sdk");
class SearchSM extends command_1.Command {
    constructor() {
        super(...arguments);
        this.searchResults = [];
    }
    async getSearchResults(searchTerm, sm, nextToken) {
        var _a, _b, _c;
        const { flags } = this.parse(SearchSM);
        const searchParams = {
            NextToken: nextToken,
            MaxResults: (_a = flags['max-results']) !== null && _a !== void 0 ? _a : 50,
        };
        const matches = await sm.listSecrets(searchParams).promise();
        let filteredList = (_b = matches.SecretList) !== null && _b !== void 0 ? _b : [];
        if (!flags.all && searchTerm) {
            filteredList = filteredList.filter((item) => { var _a; return (_a = item.Name) === null || _a === void 0 ? void 0 : _a.includes(searchTerm); });
        }
        const parameters = (_c = filteredList.map((parameter) => ({ name: parameter.Name }))) !== null && _c !== void 0 ? _c : [];
        this.searchResults.push(...parameters);
        if (matches.NextToken && (flags['no-limit'] || flags.all)) {
            await this.getSearchResults(searchTerm, sm, matches.NextToken);
        }
    }
    async promptForSecrets() {
        return inquirer.prompt([
            {
                name: 'secrets',
                message: 'Select your secrets',
                type: 'checkbox',
                choices: this.searchResults,
            },
        ]);
    }
    async run() {
        var _a, _b, _c, _d;
        const { flags } = this.parse(SearchSM);
        const sm = new aws_sdk_1.SecretsManager({ region: (_b = (_a = flags.region) !== null && _a !== void 0 ? _a : process.env.AWS_REGION) !== null && _b !== void 0 ? _b : 'us-east-1' });
        if (!((_c = sm.config.credentials) === null || _c === void 0 ? void 0 : _c.sessionToken) && process.env.NODE_ENV !== 'test') {
            this.error('Please authenticate against AWS to use this tool');
        }
        let searchTerm;
        if (!flags.all) {
            searchTerm = (_d = flags.term) !== null && _d !== void 0 ? _d : (await cli_ux_1.default.prompt('What term do you want to search for?', { type: 'normal' }));
        }
        cli_ux_1.default.action.start('listing matching parameters');
        await this.getSearchResults(searchTerm, sm);
        cli_ux_1.default.action.stop();
        if (this.searchResults && this.searchResults.length > 0) {
            let chosenSecrets = await this.promptForSecrets();
            while (chosenSecrets.secrets.length === 0) {
                this.log('No secrets selected');
                chosenSecrets = await this.promptForSecrets();
            }
            const secretRequestPromises = chosenSecrets.secrets.map((item) => {
                const getParams = {
                    SecretId: item,
                };
                return sm.getSecretValue(getParams).promise();
            });
            cli_ux_1.default.action.start('retrieving secrets');
            const result = await Promise.all(secretRequestPromises);
            cli_ux_1.default.action.stop();
            if (result && result.length > 0) {
                this.log('\n');
                cli_ux_1.default.table(result, {
                    Name: {},
                    SecretString: { header: 'Value' },
                    SecretBinary: { extended: true },
                    ARN: { extended: true },
                    VersionId: { extended: true },
                    CreatedDate: { extended: true },
                }, Object.assign({}, flags));
                return;
            }
        }
        this.warn('No Secrets found');
        this.exit(1);
    }
}
exports.default = SearchSM;
SearchSM.description = 'search parameters in AWS Secrets Manager';
SearchSM.examples = [
    `$ asap sm -t awesome-param
awesome param value
`,
];
SearchSM.flags = {
    help: command_1.flags.help({ char: 'h' }),
    term: command_1.flags.string({ exclusive: ['all'], char: 't', description: 'term to search' }),
    region: command_1.flags.string({ char: 'r', description: 'AWS region override' }),
    'max-results': command_1.flags.integer({
        exclusive: ['no-limit', 'all'],
        char: 'n',
        description: 'limit for amount of results returned, default is 50',
    }),
    'no-limit': command_1.flags.boolean({
        exclusive: ['max-results', 'all'],
        description: 'retrieve every parameter for the search term',
    }),
    all: command_1.flags.boolean({
        exclusive: ['max-results', 'no-limit', 'term'],
        description: 'retrieve every parameter in the region',
    }),
    csv: command_1.flags.boolean({ exclusive: ['no-truncate'], description: 'output is csv format' }),
    extended: command_1.flags.boolean({ char: 'x', description: 'show extra columns' }),
    'no-truncate': command_1.flags.boolean({ exclusive: ['csv'], description: 'do not truncate output to fit screen' }),
    'no-header': command_1.flags.boolean({ exclusive: ['csv'], description: 'hide table header from output' }),
};
